@@ -49,6 +49,11 @@ $(document).ready(function () {
     return this.minutes(Math[how](this.minutes() / 10) * 10).seconds(0).seconds(0).milliseconds(0)
   }
 
+  // Save date range in local storage
+  function saveDateRange(start, end) {
+    localStorage.setItem('dateRange', JSON.stringify({start: start, end: end}))
+  }
+
   // set the date range across graphs and date range picker
   function setDateRange(start, end) {
     // ensure start and end are moments
@@ -80,7 +85,14 @@ $(document).ready(function () {
       }
     })
     // save date range in local storage
-    localStorage.setItem('dateRange', JSON.stringify({start: start, end: end}))
+    saveDateRange(start, end)
+  }
+
+  function getDateRange() {
+    const daterangepicker = $('#reportrange').data('daterangepicker')
+    const start = daterangepicker.startDate.format('YYYY-MM-DD HH:mm')
+    const end = daterangepicker.endDate.format('YYYY-MM-DD HH:mm')
+    return {start: start, end: end}
   }
 
   // initialise the date range picker
@@ -115,15 +127,6 @@ $(document).ready(function () {
     }
   })
 
-  try {
-    // restore the previously saved date range
-    const data = JSON.parse(localStorage.getItem('dateRange'))
-    setDateRange(data.start, data.end)
-  } catch (error) {
-    // set a default date range
-    setDateRange(moment().subtract(2, 'days'), moment().add(1, 'hour').round10Minutes('ceil'))
-  }
-
   const defaultColors = [
     '#00cc00', '#0066b3', '#ff8000', '#dbc300', '#330099', '#990099',
     '#bce617', '#ff0000', '#808080', '#008f00', '#00487d', '#b35a00',
@@ -291,14 +294,13 @@ $(document).ready(function () {
     })
   }
 
-  // check if the axis match the data range and load more data as needed
+  // check if we have been flagged to update the data series for the plots
   function checkDataUpdates() {
     try {
       if (updatedata) {
         updatedata = false
         // go over all plots
-        $('.myplot').each(function () {
-          const plot = this
+        $('.myplot').each(function (index, plot) {
           if (plot.layout) {
             loadGraph(plot)
           }
@@ -322,6 +324,7 @@ $(document).ready(function () {
   }
   setTimeout(checkNewData, 60000)
 
+  // add a graph to the list of graphs
   function addGraph(graph, size = 'sm') {
     const clone = $('#template>:first-child').clone()
     const plot = clone.find('.myplot')[0]
@@ -380,10 +383,8 @@ $(document).ready(function () {
       layout.yaxis.exponentformat = 'E'
     }
     // get x axis zoom from date range selector
-    const daterangepicker = $('#reportrange').data('daterangepicker')
-    layout.xaxis.range = [
-      daterangepicker.startDate.format('YYYY-MM-DD HH:mm'),
-      daterangepicker.endDate.format('YYYY-MM-DD HH:mm')]
+    const range = getDateRange()
+    layout.xaxis.range = [range.start, range.end]
     // prepare the data series configuration
     const traces = []
     const tracebyfield = {}
@@ -656,6 +657,65 @@ $(document).ready(function () {
     $('#dashboards button.dropdown-toggle span').text('Dashboards')
   })
 
+  // JSON serialise the data, compress and BASE64 encode
+  function compressData(data) {
+    const stream = new window.CompressionStream('deflate')
+    const writer = stream.writable.getWriter()
+    writer.write(new TextEncoder().encode(JSON.stringify(data)))
+    writer.close()
+    return new Response(stream.readable).arrayBuffer().then(
+      compressed => btoa(Array.from(new Uint8Array(compressed)).map(b => String.fromCharCode(b)).join('')))
+  }
+
+  // BASE64 decode, decompress and JSON deserialise the data
+  function decompressData(text) {
+    const stream = new window.DecompressionStream('deflate')
+    const writer = stream.writable.getWriter()
+    writer.write(Uint8Array.from(atob(text), x => x.charCodeAt(0)).buffer)
+    writer.close()
+    return new Response(stream.readable).arrayBuffer().then(
+      buffer => JSON.parse(new TextDecoder().decode(buffer))
+    )
+  }
+
+  // make the provided dashboard active
+  function setDashboard(dashboard) {
+    if (dashboard.dateRange) {
+      setDateRange(dashboard.dateRange.start, dashboard.dateRange.end)
+    }
+    setGraphs(dashboard.graphs)
+    if (dashboard.name) {
+      $('#dashboards button.dropdown-toggle span').text(dashboard.name)
+    }
+  }
+
+  // output readable but compact JSON
+  function getCurrentDashboard() {
+    let value = '{\n'
+    const name = $('#saveDashboardName').val()
+    if (name) {
+      value += '  "name": ' + JSON.stringify(name) + ',\n'
+    }
+    if ($('#saveDashboardDateRange:checked').length) {
+      const range = getDateRange()
+      value += '  "dateRange": ' + JSON.stringify({start: range.start, end: range.end}) + ',\n'
+    }
+    value += '  "graphs": ' + JSON.stringify(getCurrentGraphs()) + '\n}'
+    return value
+  }
+
+  // get dashboard data from URL
+  function loadDashboardFromHash() {
+    if (window.location.hash.length > 2) {
+      // get list of graphs from URL
+      decompressData(window.location.hash.slice(1)).then(function (dashboard) {
+        setDashboard(dashboard)
+      })
+      // remove the hash from the URL
+      history.replaceState(null, '', ' ')
+    }
+  }
+
   // configure the dashboards button
   $.getJSON('dashboards', function (dashboards) {
     if (Object.keys(dashboards).length === 0) {
@@ -665,12 +725,7 @@ $(document).ready(function () {
         const option = $('<button class="dropdown-item" type="button">').text(name)
         $('#dashboards .dropdown-menu').append(option)
         option.click(function () {
-          const dashboard = dashboards[name]
-          setGraphs(dashboard.graphs)
-          if (dashboard.dateRange) {
-            setDateRange(dashboard.dateRange.start, dashboard.dateRange.end)
-          }
-          $('#dashboards button.dropdown-toggle span').text(name)
+          setDashboard(dashboards[name])
         })
       })
       // add import action
@@ -680,20 +735,6 @@ $(document).ready(function () {
     }
   })
 
-  // load information on available graphs
-  $.getJSON('graphs', function (data) {
-    document.graph_data = data
-    updateSelect(data)
-    // hide loading indicator and show normal interface
-    $('.loadingrow').hide()
-    $('.addgraph').show()
-    $('nav .d-none').removeClass('d-none')
-    // restore previous list of graphs
-    try {
-      setGraphs(JSON.parse(localStorage.getItem('shownGraphs')))
-    } catch (error) {}
-  })
-
   // handle showing and hiding of the save dashboard dialog
   document.getElementById('saveDashboard').addEventListener('shown.bs.modal', function () {
     $('#saveDashboardName').trigger('focus')
@@ -703,21 +744,32 @@ $(document).ready(function () {
     $('#saveDashboard .alert').remove()
   })
 
-  // output readable but compact JSON
-  function getSaveDashboardData() {
-    let value = '{\n'
-    const name = $('#saveDashboardName').val()
-    if (name) {
-      value += '  "name": ' + JSON.stringify(name) + ',\n'
-    }
-    if ($('#saveDashboardDateRange:checked').length) {
-      const dateRange = JSON.parse(localStorage.getItem('dateRange'))
-      value += '  "dateRange": ' + JSON.stringify(dateRange) + ',\n'
-    }
-    const graphs = JSON.parse(localStorage.getItem('shownGraphs'))
-    value += '  "graphs": ' + JSON.stringify(graphs) + '\n}'
-    return value
-  }
+  // save dashboard data URL to clipboard
+  $('#saveDashboardUrl').on('click', function (event) {
+    // create a temporary text area
+    const textarea = document.createElement('textarea')
+    $(textarea).css({
+      position: 'absolute',
+      left: '-1000px',
+      top: '-1000px'
+    })
+    // fill text area with URL
+    compressData(JSON.parse(getCurrentDashboard())).then(function (data) {
+      textarea.value = window.location.href.split('#')[0] + '#' + data
+      // copy text area to clipboard
+      $('#saveDashboard form').append(textarea)
+      textarea.select()
+      textarea.setSelectionRange(0, 99999)
+      document.execCommand('copy')
+      $('#saveDashboard form textarea').remove()
+      // show notification
+      $('<div class="alert alert-success">Copied to clipboard</div>').hide().appendTo('#saveDashboard .modal-body').show(200, function () {
+        setTimeout(function () {
+          bootstrap.Modal.getInstance(document.getElementById('saveDashboard')).hide()
+        }, 600)
+      })
+    })
+  })
 
   // save dialog copy data to clipboard
   $('#saveDashboardClipboard').on('click', function (event) {
@@ -729,7 +781,7 @@ $(document).ready(function () {
       top: '-1000px'
     })
     // fill text area with JSON
-    textarea.value = getSaveDashboardData()
+    textarea.value = getCurrentDashboard()
     // copy text area to clipboard
     $('#saveDashboard form').append(textarea)
     textarea.select()
@@ -747,7 +799,7 @@ $(document).ready(function () {
   // save dashboard definition to file
   $('#saveDashboardFile').on('click', function (event) {
     const element = document.createElement('a')
-    element.setAttribute('href', 'data:application/json;charset=utf-8,' + encodeURIComponent(getSaveDashboardData() + '\n'))
+    element.setAttribute('href', 'data:application/json;charset=utf-8,' + encodeURIComponent(getCurrentDashboard() + '\n'))
     element.setAttribute('download', 'dashboard.json')
     element.style.display = 'none'
     document.body.appendChild(element)
@@ -783,13 +835,7 @@ $(document).ready(function () {
     try {
       const dashboard = JSON.parse($('#loadDashboardData').val())
       if (dashboard) {
-        setGraphs(dashboard.graphs)
-        if (dashboard.dateRange) {
-          setDateRange(dashboard.dateRange.start, dashboard.dateRange.end)
-        }
-        if (dashboard.name) {
-          $('#dashboards button.dropdown-toggle span').text(dashboard.name)
-        }
+        setDashboard(dashboard)
         bootstrap.Modal.getInstance(document.getElementById('loadDashboard')).hide()
       }
     } catch (e) {
@@ -797,4 +843,36 @@ $(document).ready(function () {
       $('<div class="alert alert-danger"></div>').text('Error: ' + e.name + ': ' + e.message).hide().appendTo('#loadDashboard .modal-body').show(200)
     }
   })
+
+  try {
+    // restore the previously saved date range
+    const data = JSON.parse(localStorage.getItem('dateRange'))
+    setDateRange(data.start, data.end)
+  } catch (error) {
+    // set a default date range
+    setDateRange(moment().subtract(2, 'days'), moment().add(1, 'hour').round10Minutes('ceil'))
+  }
+
+  // load information on available graphs
+  $.getJSON('graphs', function (data) {
+    document.graph_data = data
+    updateSelect(data)
+    // hide loading indicator and show normal interface
+    $('.loadingrow').hide()
+    $('.addgraph').show()
+    $('nav .d-none').removeClass('d-none')
+    if (window.location.hash.length > 2) {
+      loadDashboardFromHash()
+    } else {
+      // restore previous list of graphs
+      try {
+        setGraphs(JSON.parse(localStorage.getItem('shownGraphs')))
+      } catch (error) {}
+    }
+  })
+
+  // handle changes to hash
+  window.addEventListener('hashchange', function () {
+    loadDashboardFromHash()
+  })
 })
